@@ -24,12 +24,10 @@ from chemicals.elements import (
     charge_from_formula
     )
 from . import Chemical
-from .utils import auom, cod_test_stoichiometry, electron_acceptor_cod
+from .utils import auom, copy_attr, cod_test_stoichiometry, electron_acceptor_cod
 
 __all__ = ('Component',)
 
-isinstance = isinstance
-
 _chemical_fields = tmo._chemical._chemical_fields
 _checked_properties = tmo._chemical._checked_properties
 lock_phase = tmo._chemical.lock_phase
@@ -61,22 +59,22 @@ _key_component_properties = ('particle_size', 'degradability', 'organic',
                              *_num_component_properties)
 
 # All Component-related properties
-_component_properties = ('measured_as', 'description', 
+_component_properties = ('measured_as', 'description',
                          *_key_component_properties)
-                         
+
 _component_slots = (*Chemical.__slots__,
                     *tuple('_'+i for i in _component_properties))
 
 _checked_properties = (*_checked_properties, *_key_component_properties)
 
 component_units_of_measure = {
-    'i_C': auom('g'), 
-    'i_N': auom('g'), 
-    'i_P': auom('g'), 
-    'i_K': auom('g'), 
-    'i_Mg': auom('g'), 
-    'i_Ca': auom('g'), 
-    'i_mass': auom('g'), 
+    'i_C': auom('g'),
+    'i_N': auom('g'),
+    'i_P': auom('g'),
+    'i_K': auom('g'),
+    'i_Mg': auom('g'),
+    'i_Ca': auom('g'),
+    'i_mass': auom('g'),
     'i_charge': auom('mol'),
     'i_COD': auom('g'),
     'i_NOD': auom('g'),
@@ -96,9 +94,9 @@ allowed_values = {
 def check_return_property(name, value):
     if name.startswith(('i_', 'f_')):
         try: return float(value)
-        except: 
-            if not value: return None            
-            raise TypeError(f'{name} must be a number, not a {type(value).__name__}.')        
+        except:
+            if not value: return None
+            raise TypeError(f'{name} must be a number, not a {type(value).__name__}.')
         if name.startswith('f_') and (value>1 or value<0):
             raise ValueError(f'{name} must be within [0,1].')
     elif name in allowed_values.keys():
@@ -150,15 +148,15 @@ class Component(Chemical):
         `degradability`, and `formula` if not given.
     f_BOD5_COD : float
         BOD5 fraction in COD of the component, unitless.
-    f_uBOD_COD : float        
+    f_uBOD_COD : float
         Ultimate BOD fraction in COD of the component, unitless.
-    f_Vmass_Totmass : float        
+    f_Vmass_Totmass : float
         Volatile fraction of the mass of the component, unitless.
     description : str
         Description of the component.
     measured_as : str
         The unit as which the component is measured.
-        Can be left as blank or chosen from 'COD', or a constituent 
+        Can be left as blank or chosen from 'COD', or a constituent
         element of the component.
     particle_size : str
         Size of the component based on the type.
@@ -172,32 +170,32 @@ class Component(Chemical):
         Will be passed to :class:`thermosteam.Chemical`.
 
     .. note::
-        
+
         [1] Element ratios like `i_C`, `i_N`, `i_P`, `i_K`, `i_Mg`, and `i_Ca` will
         be calculated based on `formula` and `measured_as` if given; and the ratio
         will be 1 if the component is measured as this element.
-        
+
         [2] For fractions including `f_BOD5_COD`, `f_uBOD_COD`, and `f_Vmass_Totmass`,
         their values must be within [0, 1].
-        
+
         [3] If no formula or MW is provided, then MW of this component is assumed to
         1 to avoid ZeroDivisionError exception in calculation.
-    
-    
+
+
     Examples
     --------
     `Component and WasteStream <https://qsdsan.readthedocs.io/en/latest/tutorials/Component_and_WasteStream.html>`_
-    
-    
+
+
     See Also
     --------
     `thermosteam.Chemical <https://thermosteam.readthedocs.io/en/latest/Chemical.html>`_
-    '''         
+    '''
 
     __slots__ = _component_slots
 
     # ID must be provided
-    def __new__(cls, ID, search_ID=None, formula=None, phase=None, measured_as=None, 
+    def __new__(cls, ID, search_ID=None, formula=None, phase=None, measured_as=None,
                 i_C=None, i_N=None, i_P=None, i_K=None, i_Mg=None, i_Ca=None,
                 i_mass=None, i_charge=None, i_COD=None, i_NOD=None,
                 f_BOD5_COD=None, f_uBOD_COD=None, f_Vmass_Totmass=None,
@@ -223,7 +221,7 @@ class Component(Chemical):
         self.i_K = i_K
         self.i_Mg = i_Mg
         self.i_Ca = i_Ca
-        self.i_charge = i_charge        
+        self.i_charge = i_charge
         self.f_BOD5_COD = f_BOD5_COD
         self.f_uBOD_COD = f_uBOD_COD
         self.f_Vmass_Totmass = f_Vmass_Totmass
@@ -290,7 +288,7 @@ class Component(Chemical):
     @i_Mg.setter
     def i_Mg(self, i):
         self._i_Mg = self._atom_frac_setter('Mg', i)
-        
+
     @property
     def i_Ca(self):
         '''[float] Calcium content of the component, [g Ca/g measure unit].'''
@@ -311,7 +309,7 @@ class Component(Chemical):
             else:
                 if self.measured_as in self.atoms:
                     i = 1/get_mass_frac(self.atoms)[self.measured_as]
-                elif self.measured_as == 'COD':                 
+                elif self.measured_as == 'COD':
                     chem_MW = molecular_weight(self.atoms)
                     chem_charge = charge_from_formula(self.formula)
                     Cr2O7 = - cod_test_stoichiometry(self.atoms, chem_charge)['Cr2O7-2']
@@ -321,12 +319,12 @@ class Component(Chemical):
                     raise AttributeError(f'Must specify i_mass for component {self.ID} '
                                          f'measured as {self.measured_as}.')
         if self.measured_as == None:
-            if i and i != 1: 
+            if i and i != 1:
                 raise AttributeError(f'Component {self.ID} is measured as itself, '
                                      f'i_mass cannot be set to values other than 1.')
             i = 1
         self._i_mass = check_return_property('i_mass', i)
-    
+
     #!!! Need to enable calculation from formula and water chemistry equilibria
     @property
     def i_charge(self):
@@ -381,7 +379,7 @@ class Component(Chemical):
     @f_Vmass_Totmass.setter
     def f_Vmass_Totmass(self, f):
         self._f_Vmass_Totmass = check_return_property('f_Vmass_Totmass', f)
-        
+
     @property
     def description(self):
         '''[str] Description of the component.'''
@@ -394,31 +392,31 @@ class Component(Chemical):
     def measured_as(self):
         '''
         [str] The unit as which the component is measured.
-        Can be left as blank or chosen from 'COD', or a constituent 
+        Can be left as blank or chosen from 'COD', or a constituent
         element of the component.
         '''
         return self._measured_as
     @measured_as.setter
     def measured_as(self, measured_as):
         '''
-        When measured_as is set to a different value, all i_{} values will 
+        When measured_as is set to a different value, all i_{} values will
         be automatically updated.
         '''
         if measured_as:
-            if measured_as == 'COD': 
+            if measured_as == 'COD':
                 self._MW = molecular_weight({'O':2})
             elif measured_as in self.atoms:
                 self._MW = molecular_weight({measured_as:1})
             else:
                 raise AttributeError(f"Component {self.ID} must be measured as "
                                      f"either COD or one of its constituent atoms, "
-                                     f"if not as itself.")        
-        
+                                     f"if not as itself.")
+
         if self._measured_as != measured_as:
             self._convert_i_attr(measured_as)
-            
+
         self._measured_as = measured_as
-        
+
     def _convert_i_attr(self, new):
         if new == None:
             denom = self._i_mass
@@ -432,15 +430,15 @@ class Component(Chemical):
             raise AttributeError(f"Component {self.ID} must be measured as "
                                  f"either COD or one of its constituent atoms, "
                                  f"if not as itself.")
-        
+
         if denom == 0:
             raise ValueError(f'{self.ID} cannot be measured as {new}')
-        
+
         for field in _num_component_properties:
             if field.startswith('i_'):
                 new_i = getattr(self, '_'+field)/denom
-                setattr(self, '_'+field, new_i)            
-    
+                setattr(self, '_'+field, new_i)
+
     @property
     def particle_size(self):
         '''
@@ -479,14 +477,14 @@ class Component(Chemical):
     @i_COD.setter
     def i_COD(self, i):
         if i: self._i_COD = check_return_property('i_COD', i)
-        else:             
-            if self.organic or self.formula in ('H2', 'O2', 'N2', 'NO2-', 'NO3-'): 
+        else:
+            if self.organic or self.formula in ('H2', 'O2', 'N2', 'NO2-', 'NO3-'):
                 if self.measured_as == 'COD': self._i_COD = 1.
                 elif not self.atoms:
                     raise AttributeError(f"Must specify `i_COD` for organic component {self.ID}, "
                                          f"which is not measured as COD and has no formula.")
                 else:
-                    chem_MW = molecular_weight(self.atoms) 
+                    chem_MW = molecular_weight(self.atoms)
                     chem_charge = charge_from_formula(self.formula)
                     if self.formula in ('O2', 'N2', 'NO2-', 'NO3-'):
                         cod = electron_acceptor_cod(self.atoms, chem_charge) * molecular_weight({'O':2})
@@ -494,14 +492,14 @@ class Component(Chemical):
                         Cr2O7 = - cod_test_stoichiometry(self.atoms, chem_charge)['Cr2O7-2']
                         cod = Cr2O7 * 1.5 * molecular_weight({'O':2})
                     self._i_COD = check_return_property('i_COD', cod/chem_MW * self.i_mass)
-            else: self._i_COD = 0. 
+            else: self._i_COD = 0.
 
     @property
     def i_NOD(self):
         '''
         [float] Nitrogenous oxygen demand, calculated based on `measured_as`,
         `degradability`, and `formula` if not given.'''
-        return self._i_NOD or 0.        
+        return self._i_NOD or 0.
     @i_NOD.setter
     def i_NOD(self, i):
         if i == None:
@@ -512,7 +510,7 @@ class Component(Chemical):
             else:
                 i = 0.
         self._i_NOD = check_return_property('i_NOD', i)
-            
+
 
     def show(self, chemical_info=False):
         '''
@@ -538,12 +536,12 @@ class Component(Chemical):
             if value is None:
                 line = f"{field}: None"
             elif str(value) in ('True', 'False'):
-                line = f"{field}: {value}" 
+                line = f"{field}: {value}"
             else:
                 if isinstance(value, (int, float)):
                     line = f"{field}: {value:.5g}"
                     units = component_units_of_measure.get(field, '')
-                    if units: 
+                    if units:
                         if field.startswith('i_'):
                             if self._measured_as: denom = self._measured_as
                             else: denom = ''
@@ -557,7 +555,7 @@ class Component(Chemical):
             section.append(line)
         info += header + ("\n" + 9*" ").join(section)
         print(info)
-        
+
     _ipython_display_ = show
 
     def get_missing_properties(self, properties=None):
@@ -578,13 +576,7 @@ class Component(Chemical):
         alternative data set by kwargs.
         '''
         new = self.__class__.__new__(cls=self.__class__, ID=new_ID)
-
-        for field in self.__slots__:
-            #!!! Take '_other_names' out after thermosteam is updated
-            if field in ('_CAS', '_N_solutes', '_other_names'): continue
-            value = getattr(self, field)
-            setattr(new, field, copy_maybe(value))
-
+        new = copy_attr(new, self, skip=('_ID', '_CAS', '_N_solutes'))
         new._ID = new_ID
         new._locked_state = self._locked_state
         new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus, new.Tm,
@@ -592,14 +584,15 @@ class Component(Chemical):
         new._label_handles()
         for i,j in data.items(): setattr(new, i , j)
         return new
+
     __copy__ = copy
 
     @classmethod
-    def from_chemical(cls, ID, chemical=None, phase=None, measured_as=None, 
+    def from_chemical(cls, ID, chemical=None, phase=None, measured_as=None,
                       i_C=None, i_N=None, i_P=None, i_K=None, i_Mg=None, i_Ca=None,
                       i_mass=None, i_charge=None, i_COD=None, i_NOD=None,
-                      f_BOD5_COD=None, f_uBOD_COD=None, f_Vmass_Totmass=None, 
-                      description=None, particle_size=None, degradability=None, 
+                      f_BOD5_COD=None, f_uBOD_COD=None, f_Vmass_Totmass=None,
+                      description=None, particle_size=None, degradability=None,
                       organic=None, **data):
         '''Return a new :class:`Component` from a :class:`thermosteam.Chemical` object.'''
         new = cls.__new__(cls, ID=ID, phase=phase)
@@ -620,7 +613,7 @@ class Component(Chemical):
         new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus, new.Tm,
                            new.Tb, new.eos, new.eos_1atm, new.phase_ref)
         new._label_handles()
-        new._measured_as = measured_as        
+        new._measured_as = measured_as
         new.i_mass = i_mass
         new.i_C = i_C
         new.i_N = i_N
@@ -642,22 +635,4 @@ class Component(Chemical):
             if i == 'formula':
                 new._formula = j
             else: setattr(new, i , j)
-        return new
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+        return new
\ No newline at end of file
