@@ -8,7 +8,7 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
@@ -18,11 +18,14 @@ for license details.
 import math
 import numpy as np
 import pandas as pd
-from . import ImpactItem, WasteStream
-from ._units_of_measure import auom
-from .utils.formatting import format_number as f_num
+from collections.abc import Iterable
+from warnings import warn
+from . import ImpactItem, Stream, SanStream, SanUnit
+from .utils import (
+    auom,
+    format_number as f_num
+    )
 
-items = ImpactItem._items
 isinstance = isinstance
 iter = iter
 callable = callable
@@ -38,16 +41,148 @@ class LCA:
     ----------
     system : :class:`biosteam.System`
         System for which this LCA is conducted for.
-    lifetime : float
+    lifetime : int
         Lifetime of the LCA.
     lifetime_unit : str
         Unit of lifetime.
     uptime_ratio : float
-        Fraction of time that the plant is operating.
+        Fraction of time that the system is operating.
     item_quantities : kwargs, :class:`ImpactItem` or str = float/callable or (float/callable, unit)
         Other :class:`ImpactItem` objects (e.g., electricity) and their quantities.
         Note that callable functions are used so that quantity of items can be updated.
     
+
+    Examples
+    --------
+    A system should be constructed prior to LCA, here we import a pre-constructed one.
+    
+    >>> import qsdsan as qs
+    >>> from qsdsan.utils import load_example_cmps, load_example_sys
+    >>> cmps = load_example_cmps()
+    >>> sys = load_example_sys(cmps)
+    >>> # Uncomment the line below to see the system diagram
+    >>> # sys.diagram()
+    >>> sys.simulate()
+    >>> sys.show()
+    System: sys
+    ins...
+    [0] water
+        phase: 'l', T: 298.15 K, P: 101325 Pa
+        flow (kmol/hr): H2O   111
+                        NaCl  0.856
+    [1] methanol
+        phase: 'l', T: 298.15 K, P: 101325 Pa
+        flow (kmol/hr): Methanol  0.624
+    [2] ethanol
+        phase: 'l', T: 298.15 K, P: 101325 Pa
+        flow (kmol/hr): Ethanol  0.217
+    outs...
+    [0] alcohols
+        phase: 'l', T: 298.15 K, P: 101325 Pa
+        flow (kmol/hr): Methanol  0.624
+                        Ethanol   0.217
+    [1] waste_brine
+        phase: 'l', T: 350 K, P: 101325 Pa
+        flow (kmol/hr): H2O   88.8
+                        NaCl  0.684
+    
+    And we also need to specify the impact indicators that we are interested in.
+    
+    >>> GWP = qs.ImpactIndicator('GlobalWarming', alias='GWP', unit='kg CO2-eq')
+    >>> FEC = qs.ImpactIndicator('FossilEnergyConsumption', alias='FEC', unit='MJ')
+    
+    There are four different types of impacts in `QSDsan`:
+    construction, transportation, stream, and others.
+    
+    Note that it is best to add the impact items when developing the unit module,
+    (i.e., typically in the `_design` function, but can also in `_run`  or `_cost`)
+    but for illustrative purpose, we add it after the system is constructed.
+    
+    Construction is mainly used for impacts that only occur once per lifetime
+    of the equipment or the unit.
+    
+    For example, assume we want to consider the amount of stainless steel
+    and concrete used in constructing the MixTank M1.
+    
+    >>> # Make the impact item, numbers are made up
+    >>> SS = qs.ImpactItem('SS', functional_unit='kg', GWP=3, FEC=50)
+    >>> Concrete = qs.ImpactItem('Concrete', functional_unit='kg', GWP=4, FEC=30)
+    >>> # Specify the amount of stainless steel and concrete used in the unit
+    >>> SS_constr_M1 = qs.Construction(item=SS, quantity=100)
+    >>> Concrete_constr_M1 = qs.Construction(item=Concrete, quantity=50)
+    >>> # Retrieve the unit from the registry
+    >>> flowsheet = qs.Flowsheet.flowsheet.default
+    >>> M1 = flowsheet.unit.M1
+    >>> # Add the construction activity
+    >>> M1.construction = (SS_constr_M1, Concrete_constr_M1)
+    
+    Transportation activity can be added in a similar manner, assuming that
+    stainless steel and concrete are delivered by truck from 500 km away.
+
+    The interval set below is assuming a system lifetime of 10 year
+    and this delivery is only needed once for the entire lifetime.
+    
+    >>> lifetime = 10
+    >>> Trucking = qs.ImpactItem('Trucking', functional_unit='kg*km',
+    ...                          GWP=0.5, FEC=1.5)
+    >>> total_quantity = SS_constr_M1.quantity + Concrete_constr_M1.quantity
+    >>> Trans_M1 = qs.Transportation(item=Trucking, load_type='mass',
+    ...                              load=total_quantity, distance=500,
+    ...                              interval=lifetime, interval_unit='yr')
+    >>> M1.transportation = Trans_M1
+
+    We can als consider the impacts associated with chemicals and emissions.
+    For example, assume the acquisition of methanol, ethanol and disposal of
+    the waste brine all have impacts, but the generated alcohols can be treated
+    as a product therefore have credits with 
+    
+    >>> # Retrieve streams
+    >>> methanol = flowsheet.stream.methanol
+    >>> ethanol = flowsheet.stream.ethanol
+    >>> alcohols = flowsheet.stream.alcohols
+    >>> waste_brine = flowsheet.stream.waste_brine
+    >>> # Create `StreamImpactItem` and link to the streams
+    >>> methanol_item = qs.StreamImpactItem(linked_stream=methanol, GWP=2, FEC=13)
+    >>> ethanol_item = qs.StreamImpactItem(linked_stream=ethanol, GWP=2.1, FEC=25)
+    >>> alcohols_item = qs.StreamImpactItem(linked_stream=alcohols, GWP=-0.2, FEC=-5)
+    >>> brine_item = qs.StreamImpactItem(linked_stream=methanol, GWP=2, FEC=3)
+    
+    Finally, there might be other impacts we want to include in the LCA,
+    for example, the electricity needed to operate the system.
+    
+    We can use add those additional items when creating the `LCA` object.
+    
+    >>> # Get the electricity usage of the system throughout the lifetime,
+    >>> # note that the default power utility unit is hr
+    >>> total_power = sum((i.rate for i in sys.power_utilities))*24*365*lifetime
+    >>> # Create an impact item for the electricity
+    >>> e_item = qs.ImpactItem('e_item', 'kWh', GWP=1.1, FEC=24)
+    >>> # Create the LCA object
+    >>> lca = qs.LCA(system=sys, lifetime=10, e_item=total_power)
+
+    Now we can look at the total impacts associate with this system.
+    
+    >>> lca.show() # doctest: +SKIP
+    LCA: sys (lifetime 10 yr)
+    Impacts:
+                                  Construction  Transportation  WasteStream   Others    Total
+    FossilEnergyConsumption (MJ)       6.5e+03        1.12e+05      1.4e+07 1.95e+06 1.61e+07
+    GlobalWarming (kg CO2-eq)              500        3.75e+04     4.82e+06 8.94e+04 4.95e+06
+    >>> # Retrieve impacts associated with a specific indicator
+    >>> lca.total_impacts['GlobalWarming']
+    4945363.316130896
+    
+    Or breakdowns of the different category
+    
+    >>> lca.total_impacts['GlobalWarming'] # doctest: +ELLIPSIS
+    4945363.3...
+    >>> lca.get_impact_table('Construction') # doctest: +SKIP
+
+    See Also
+    --------
+    `SanUnit and System <https://qsdsan.readthedocs.io/en/latest/tutorials/SanUnit_and_System.html>`_
+    
+    `TEA and LCA <https://qsdsan.readthedocs.io/en/latest/tutorials/TEA_and_LCA.html>`_
     '''
     
     __slots__ = ('_system',  '_lifetime', '_uptime_ratio',
@@ -78,6 +213,8 @@ class LCA:
     
     def _update_system(self, system):
         for u in system.units:
+            if not isinstance (u, SanUnit):
+                continue
             if u.construction:
                 self._construction_units.add(u)
             if u.transportation:
@@ -87,6 +224,8 @@ class LCA:
         self._transportation_units = sorted(self._transportation_units,
                                             key=lambda u: u.ID)
         for s in (i for i in system.feeds+system.products):
+            if not hasattr(s, 'impact_item'):
+                continue
             if s.impact_item:
                 self._lca_streams.add(s)
         self._lca_streams = sorted(self._lca_streams, key=lambda s: s.ID)
@@ -95,16 +234,16 @@ class LCA:
 
     def _update_lifetime(self, lifetime=0., unit='yr'):
         if not unit or unit == 'yr':
-            self._lifetime = float(lifetime)
+            self._lifetime = int(lifetime)
         else:
-            converted = auom(unit).convert(float(lifetime), 'yr')
+            converted = auom(unit).convert(int(lifetime), 'yr')
             self._lifetime = converted
 
     
     def add_other_item(self, item, f_quantity, unit=''):
         '''Add other :class:`ImpactItem` in LCA.'''
         if isinstance(item, str):
-            item = items[item]
+            item = ImpactItem.get_item(item)
         fu = item.functional_unit
         if not callable(f_quantity):
             f = lambda: f_quantity
@@ -160,7 +299,7 @@ class LCA:
         Return all construction-related impacts for the given unit,
         normalized to a certain time frame.
         '''
-        if not (isinstance(units, tuple) or isinstance(units, list) or isinstance(units, set)):
+        if not isinstance(units, Iterable):
             units = (units,)
         if not time:
             ratio = 1
@@ -169,6 +308,8 @@ class LCA:
             ratio = converted/self.lifetime_hr
         impacts = dict.fromkeys((i.ID for i in self.indicators), 0.)
         for i in units:
+            if not isinstance(i, SanUnit):
+                continue
             for j in i.construction:
                 impact = j.impacts
                 if j.lifetime is not None:
@@ -184,8 +325,7 @@ class LCA:
         Return all transportation-related impacts for the given unit,
         normalized to a certain time frame.
         '''
-        if not (isinstance(units, tuple) or isinstance(units, list)
-                or isinstance(units, set)):
+        if not isinstance(units, Iterable):
             units = (units,)
         if not time:
             time = self.lifetime_hr
@@ -193,6 +333,8 @@ class LCA:
             time = auom(time_unit).convert(float(time), 'hr')
         impacts = dict.fromkeys((i.ID for i in self.indicators), 0.)
         for i in units:
+            if not isinstance(i, SanUnit):
+                continue
             for j in i.transportation:
                 impact = j.impacts
                 for m, n in impact.items():
@@ -206,11 +348,9 @@ class LCA:
         Return all stream-related impacts for the given streams,
         normalized to a certain time frame.
         '''
-        if not (isinstance(stream_items, tuple) or isinstance(stream_items, list)
-                or isinstance(stream_items, set)):
+        if not isinstance(stream_items, Iterable):
             stream_items = (stream_items,)
-        if not (isinstance(exclude, tuple) or isinstance(exclude, list)
-                or isinstance(exclude, set)):
+        if not isinstance(exclude, Iterable):
             exclude = (exclude,)
         if stream_items == None:
             stream_items = self.stream_inventory
@@ -221,7 +361,9 @@ class LCA:
             time = auom(time_unit).convert(float(time), 'hr')
         for j in stream_items:
             # In case that ws instead of the item is given
-            if isinstance(j, WasteStream):
+            if isinstance(j, Stream):
+                if not isinstance(j, SanStream):
+                    continue
                 ws = j
                 if j.impact_item:
                     j = ws.impact_item
@@ -251,7 +393,7 @@ class LCA:
         impacts = dict.fromkeys((i.ID for i in self.indicators), 0.)
         other_dct = self.other_items         
         for i in other_dct.keys():
-            item = items[i]
+            item = ImpactItem.get_item(i)
             for m, n in item.CFs.items():
                 impacts[m] += n*other_dct[i]['quantity']
         return impacts
@@ -275,16 +417,16 @@ class LCA:
         
         Parameters
         ----------
-        streams : :class:`WasteStream` or sequence
-            One or a sequence of streams. Note that impacts of these streams will be
+        streams : :class:`WasteStream` or iterable
+            One or a iterable of streams. Note that impacts of these streams will be
             excluded in calculating the total impacts.
-        allocate_by : str, sequence, or function to generate an sequence
+        allocate_by : str, iterable, or function to generate an iterable
             If provided as a str, can be "mass", "energy", or 'value' to allocate
             the impacts accordingly.
-            If provided as a sequence (no need to normalize so that sum of the sequence is 1),
-            will allocate impacts according to the sequence.
+            If provided as an iterable (no need to normalize so that sum of the iterable is 1),
+            will allocate impacts according to the iterable.
             If provided as a function,  will call the function to generate an
-            sequence to allocate the impacts accordingly.
+            iterable to allocate the impacts accordingly.
         
         .. note::
             
@@ -292,13 +434,14 @@ class LCA:
             in the stream.
         
         '''
-        if not (isinstance(streams, tuple) or isinstance(streams, list)
-                or isinstance(streams, set)):
+        if not isinstance(streams, Iterable):
             streams = (streams,)
         impact_dct = self.get_total_impacts(exclude=streams)
         impact_vals = np.array([i for i in impact_dct.values()])
         allocated = {}
         if len(streams) == 1:
+            if not isinstance(streams[0], SanStream):
+                return None
             return impact_dct
         if allocate_by == 'mass':
             ratios = np.array([i.F_mass for i in streams])
@@ -312,36 +455,37 @@ class LCA:
             ratios = allocate_by()
         else:
             raise ValueError('allocate_by can only be "mass", "energy", "value", '
-                             'a sequence, or a function to generate a sequence.')
+                             'an iterable, or a function to generate an iterable.')
         if ratios.sum() == 0:
             raise ValueError('Calculated allocation ratios are all zero, cannot allocate.')
         ratios = ratios/ratios.sum()
-        for n, ws in enumerate(streams):
-            if not ws in self.system.streams:
-                raise ValueError(f'`WasteStream` {ws} not in the system.')
-            allocated[ws.ID] = dict.fromkeys(impact_dct.keys(),
-                                             (ratios[n]*impact_vals).sum())
+        for n, s in enumerate(streams):
+            if not isinstance(s, SanStream):
+                continue
+            if not s in self.system.streams:
+                raise ValueError(f'`WasteStream` {s} not in the system.')
+            allocated[s.ID] = dict.fromkeys(impact_dct.keys(),
+                                            (ratios[n]*impact_vals).sum())
         return allocated
         
     
     def get_unit_impacts(self, units, time=None, time_unit='hr',
                           exclude=None):
         '''Return total impacts with certain units, normalized to a certain time frame. '''
-        if not (isinstance(units, tuple) or isinstance(units, list)
-                or isinstance(units, set)):
+        if not isinstance(units, Iterable):
             units = (units,)
         constr = self.get_construction_impacts(units, time, time_unit)
         trans = self.get_transportation_impacts(units, time, time_unit)
-        ws_items = set(i for i in 
+        stream_items = set(i for i in 
                        sum((tuple(unit.ins+unit.outs) for unit in units), ())
                        if i.impact_item)
 
-        ws = self.get_stream_impacts(stream_items=ws_items, exclude=exclude,
+        s = self.get_stream_impacts(stream_items=stream_items, exclude=exclude,
                                      time=time, time_unit=time_unit)
         other = self.get_other_impacts()
         tot = constr.copy()
         for m in tot.keys():
-            tot[m] += trans[m] + ws[m] + other[m]
+            tot[m] += trans[m] + s[m] + other[m]
         return tot
     
     def _append_cat_sum(self, cat_table, cat, tot):
@@ -359,7 +503,7 @@ class LCA:
             cat_table.rename(index={num: 'Sum'}, inplace=True)
         return cat_table
     
-    def get_impact_table(self, category=None, time=None, time_unit='hr'):
+    def get_impact_table(self, category, time=None, time_unit='hr'):
         '''
         Return a :class:`pandas.DataFrame` table for the given impact category,
         normalized to a certain time frame.
@@ -369,13 +513,13 @@ class LCA:
         else:
             time = auom(time_unit).convert(float(time), 'hr')
         
-        if category in ('Construction', 'Other'):
-            time = time/self.lifetime_hr
-        
         cat = category.lower()
         tot = getattr(self, f'total_{cat}_impacts')
-        if category in ('Construction', 'Transportation'):
-            cat = category.lower()
+        
+        if cat in ('construction', 'other'):
+            time = time/self.lifetime_hr
+
+        if cat in ('construction', 'transportation'):
             units = sorted(getattr(self, f'_{cat}_units'),
                               key=(lambda su: su.ID))
             items = sorted(set(i.item for i in getattr(self,  f'{cat}_inventory')),
@@ -388,11 +532,14 @@ class LCA:
             for item_ID in item_dct.keys():
                 item_dct[item_ID] = dict(SanUnit=[], Quantity=[])
             for su in units:
+                if not isinstance(su, SanUnit):
+                    continue
                 for i in getattr(su, cat):
                     item_dct[i.item.ID]['SanUnit'].append(su.ID)
                     item_dct[i.item.ID]['Quantity'].append(i.quantity*time)
                     if cat == 'transportation':
                         item_dct[i.item.ID]['Quantity'][-1] /= i.interval
+                        
             dfs = []
             for item in items:
                 dct = item_dct[item.ID]
@@ -420,7 +567,7 @@ class LCA:
         ind_head = sum(([f'{i.ID} [{i.unit}]',
                          f'Category {i.ID} Ratio'] for i in self.indicators), [])
         
-        if category == 'Stream':
+        if cat in ('stream', 'streams'):
             headings = ['Stream', 'Mass [kg]', *ind_head]
             item_dct = dict.fromkeys(headings)
             for key in item_dct.keys():
@@ -442,7 +589,7 @@ class LCA:
             table.set_index(['Stream'], inplace=True)
             return self._append_cat_sum(table, cat, tot)
 
-        elif category == 'Other':
+        elif cat == 'other':
             headings = ['Other', 'Quantity', *ind_head]
             item_dct = dict.fromkeys(headings)
             for key in item_dct.keys():
@@ -460,14 +607,15 @@ class LCA:
                     else:
                         item_dct[f'{ind.ID} [{ind.unit}]'].append(0)
                         item_dct[f'Category {ind.ID} Ratio'].append(0)
+            
             table = pd.DataFrame.from_dict(item_dct)
             table.set_index(['Other'], inplace=True)
             return self._append_cat_sum(table, cat, tot)
         
-        else:
-            raise ValueError(
-                'category can only be "Construction", "Transportation", "Stream", or "Other", ' \
-                f'not {category}.')
+        raise ValueError(
+            'category can only be "Construction", "Transportation", "Stream", or "Other", ' \
+            f'not "{category}".')
+
 
     def save_report(self, file=None, sheet_name='LCA',
                     time=None, time_unit='hr',
@@ -483,6 +631,7 @@ class LCA:
                 table.to_excel(writer, sheet_name=sheet_name, startrow=n_row)
                 n_row += table.shape[0] + row_space + len(table.columns.names) # extra lines for the heading
 
+
     @property
     def system(self):
         '''[biosteam.System] The system linked to this LCA.'''
@@ -493,7 +642,7 @@ class LCA:
     
     @property
     def lifetime(self):
-        '''[float] Lifetime of the system, [yr].'''
+        '''[int] Lifetime of the system, [yr].'''
         return self._lifetime
     @lifetime.setter
     def lifetime(self, lifetime, unit='yr'):
@@ -510,7 +659,7 @@ class LCA:
         return self._uptime_ratio
     @uptime_ratio.setter
     def uptime_ratio(self, i):
-        if 0 <= i <= 1:
+        if 0 <=i<= 1:
             self._uptime_ratio = float(i)
         else:
             raise ValueError('uptime_ratio must be in [0,1].')
@@ -536,10 +685,11 @@ class LCA:
         if not self.other_items:
             other = set()
         else:
-            other = set(sum((items[i].indicators for i in self.other_items.keys()), ()))
+            other = set(sum((ImpactItem.get_item(i).indicators
+                             for i in self.other_items.keys()), ()))
         tot = constr.union(trans, ws, other)
         if len(tot) == 0:
-            raise ValueError('No `ImpactIndicators` have been added.')
+            warn('No `ImpactIndicator` has been added.')
         return tot
     
     @property
