@@ -8,7 +8,7 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 
 TODO (maybe):
@@ -22,7 +22,7 @@ TODO (maybe):
 import numpy as np
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils.loading import load_data, data_path
+from ..utils import load_data, data_path
 
 __all__ = ('AnaerobicDigestion',)
 
@@ -61,9 +61,9 @@ class AnaerobicDigestion(SanUnit, Decay):
     
     '''
     
-    def __init__(self, ID='', ins=None, outs=(), flow_rate=None,
-                 if_capture_biogas=True, if_N2O_emission=False, **kwargs):
-        SanUnit.__init__(self, ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
+                 flow_rate=None, if_capture_biogas=True, if_N2O_emission=False, **kwargs):
+        SanUnit.__init__(self, ID, ins, outs, thermo, init_with)
         self._flow_rate = flow_rate
         self.if_capture_biogas = if_capture_biogas
         self.if_N2O_emission = if_N2O_emission
