@@ -8,7 +8,7 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
@@ -19,7 +19,7 @@ import numpy as np
 from warnings import warn
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils.loading import load_data, data_path
+from ..utils import load_data, data_path
 
 __all__ = ('DryingBed',)
 
@@ -54,10 +54,10 @@ class DryingBed(SanUnit, Decay):
     
     '''
     
-    def __init__(self, ID='', ins=None, outs=(), design_type='unplanted',
-                 **kwargs):
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
+                 design_type='unplanted', **kwargs):
 
-        SanUnit.__init__(self, ID, ins, outs)
+        SanUnit.__init__(self, ID, ins, outs, thermo, init_with)
         N_unplanted = {'covered': 19,
                        'uncovered': 30,
                        'storage': 19,
@@ -161,7 +161,7 @@ class DryingBed(SanUnit, Decay):
             Construction(item='Steel', quantity=steel, quantity_unit='kg'),
             )
         for i in self.construction:
-            self._BM[i.item.ID] = 1
+            self.F_BM[i.item.ID] = 1
 
     @property
     def tau(self):
