@@ -8,14 +8,14 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
 # %%
 
 from .. import SanUnit
-from ..utils.loading import load_data, data_path
+from ..utils import load_data, data_path
 
 __all__ = ('Excretion',)
 
@@ -41,8 +41,9 @@ class Excretion(SanUnit):
     _N_ins = 0
     _N_outs = 2
     
-    def __init__(self, ID='', ins=None, outs=(), **kwargs):                
-        SanUnit.__init__(self, ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
+                 **kwargs):                
+        SanUnit.__init__(self, ID, ins, outs, thermo, init_with)
         data = load_data(path=data_path)
         for para in data.index:
             value = float(data.loc[para]['expected'])
@@ -53,7 +54,6 @@ class Excretion(SanUnit):
             setattr(self, attr, value)
 
     def _run(self):
-        
         ur, fec = self.outs
         ur.empty()
         fec.empty()
