@@ -8,7 +8,7 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
@@ -19,7 +19,7 @@ import numpy as np
 from warnings import warn
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils.loading import load_data, data_path
+from ..utils import load_data, data_path
 
 __all__ = ('Lagoon',)
 
@@ -55,10 +55,10 @@ class Lagoon(SanUnit, Decay):
 
     '''
     
-    def __init__(self, ID='', ins=None, outs=(), design_type='anaerobic',
-                 flow_rate=None, if_N2O_emission=False, **kwargs):    
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
+                 design_type='anaerobic', flow_rate=None, if_N2O_emission=False, **kwargs):    
         
-        SanUnit.__init__(self, ID, ins, outs)
+        SanUnit.__init__(self, ID, ins, outs, thermo, init_with)
         self._tau = None
         self._P_removal = 0.
         
