@@ -8,7 +8,7 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
@@ -18,7 +18,7 @@ for license details.
 from warnings import warn
 from .. import SanUnit
 from ._decay import Decay
-from ..utils.loading import load_data, data_path
+from ..utils import load_data, data_path
 
 __all__ = ('SludgeSeparator',)
 
@@ -53,9 +53,10 @@ class SludgeSeparator(SanUnit):
     
     '''
     
-    def __init__(self, ID='', ins=None, outs=(), split=None, settled_frac=None):    
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
+                 split=None, settled_frac=None, **kwargs):    
         
-        SanUnit.__init__(self, ID, ins, outs)
+        SanUnit.__init__(self, ID, ins, outs, thermo, init_with, **kwargs)
         data = load_data(path=data_path)
         if not split:
             value = eval(data.loc['split']['expected'])
