@@ -8,7 +8,7 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
-Please refer to https://github.com/QSD-Group/QSDsan/blob/master/LICENSE.txt
+Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
@@ -18,7 +18,7 @@ for license details.
 import numpy as np
 from .. import Construction
 from ._toilet import Toilet
-from ..utils.loading import load_data, data_path
+from ..utils import load_data, data_path
 
 __all__ = ('UDDT',)
 
@@ -73,14 +73,15 @@ class UDDT(Toilet):
     
     '''
     
-    def __init__(self, ID='', ins=None, outs=(), N_user=1, N_toilet=1, lifetime=8,
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
+                 N_user=1, N_toilet=1, lifetime=8,
                  if_toilet_paper=True, if_flushing=True, if_cleansing=False,
                  if_desiccant=True, if_air_emission=True, if_ideal_emptying=True,
                  CAPEX=553, OPEX_over_CAPEX=0.1,
                  T=273.15+24, safety_factor=1, if_prep_loss=True, if_treatment=False,
                  **kwargs):
 
-        Toilet.__init__(self, ID, ins, outs, N_user, N_toilet,
+        Toilet.__init__(self, ID, ins, outs, thermo, init_with, N_user, N_toilet,
                         if_toilet_paper, if_flushing, if_cleansing, if_desiccant,
                         if_air_emission, if_ideal_emptying, CAPEX, OPEX_over_CAPEX)
         self.lifetime = lifetime
